we04ucc1 = function(psize,a,lower,upper,objfun,confun,initial,optlist = NA) {
  ## R code to call e04ucc
  
  ## psize   - Vector containing the problem size, psize = c(N,NCLIN,NCNLIN),
  ##           where N = number of variables, NCLIN = number linear constraints
  ##           NCNLIN = number of non-linear constraints
  ## a       - (NCLIN x N) Matrix of linear constraints. If NCLIN = 0, a is not referenced, 
  ## lower   - List of lower constraint limits. List should contain three elements, the first
  ##           a vector of N limits for simple constraints, the second a vector of NCLIN
  ##           limits for the linear constraints and the third a vector of NCNLIN limits for
  ##           the non-linear constraints
  ## upper   - List of upper constraint limits. This should be in the same form as lower
  ## objfun  - Function that calculates the objective function and its gradients. Needs to
  ##           return a list with two elements, one named "objf" and one named "objgrd" holding
  ##           the value of the objective function and its gradient respectively.
  ##           objfun = function(n,x,flag,first)
  ##               n     - N, the number of parameters
  ##               x     - Vector of length N holding the values at which the objective function
  ##                       should be evaluated
  ##               flag  - If 0 or 2 the value of the objective function should be calculated
  ##                       If 2 the value of the gradient should be calculated
  ##               first - Set to TRUE if this is the first call to the objfun
  ##           If objfun returns a list that includes an entry called "terminate" the routine will stop
  ## confun  - Function that calculates the non-linear constraints and corresponding derivatives.
  ##           Needs to return a list with two elements, one named "conf" and one named "conjac" holding
  ##           the value of the constraints and their gradients respectively.
  ##           confun = function(n,ncnlin,needc,x,flag,first)
  ##              n      - N, the number of parameters
  ##              ncnlin - NCNLIN, the number of non-linear constraints
  ##              needc  - Vector of length NCNLIN, if needc[i] = 1, then values for the ith constraint
  ##                       are required
  ##               x     - Vector of length N holding the values at which the objective function
  ##                       should be evaluated
  ##               flag  - If 0 or 2 the value of the objective function should be calculated
  ##                       If 2 the value of the gradient should be calculated
  ##               first - Set to TRUE if this is the first call to the confun
  ##           If confun returns a list that includes an entry called "terminate" the routine will stop
  ## initial - Vector of N initial values
  ## optlist - List of options. The names of the items in the list should correspond to the names of the
  ##           optional parameters as given in section 11.1 of the documentation for e04ucc.
  ##           i.e. optlist=list(list=TRUE, f_prec = 0.4, print_level = 1), sets the optional parameter
  ##           list to Nag_TRUE, the problem accuracy (f_prec) to 0.4 and the print level to Nag_Soln.
  ##
  ##           For options that are enumerations (as print_level is), an integer code should be used, starting
  ##           from 0. The options are then numbered in the same order as they appear in the "constraints" 
  ##           section of the documentation for that option, i.e. for print_level 0 = Nag_NoPrint, 1 = Nag_Soln,
  ##           2 = Nag_Iter etc.
  ##       
  ##           The following options are not catered for: start,conf,conjac,lambda and print_fun
  ##
  ##           If optlist is given as a list, irrespective of whether it has any valid option in it, then
  ##           additional output that is documented as being returned in the Nag_E04_Opt argument options
  ##           is returned
  add.missing.names = function(l,l.names) {
    ## Adds any names from l.names that are missing from l, in order (so first missing value is
    ## given the first unused name from l.names)
    ll.names = names(l)
    id.to.replace = (1:length(l))[!(ll.names%in%l.names)]
    replace.with = l.names[!(l.names%in%ll.names)]
    nmin = min(length(id.to.replace),length(replace.with))
    ll.names[id.to.replace[1:nmin]] = replace.with[1:nmin]
    names(l) = ll.names
    l
  }    
  i.objfun = function() {
    ll = objfun(n,x,objgrd,flag,first)
    ## Expect objfun to return a list with two elements, one named objf and one named objgrd
    add.missing.names(ll,c("objf","objgrd"))
  }
  i.confun = function() {
    ll = confun(n,ncnlin,needc,x,conjac,flag,first)
    ## Expect confun to return a list with two elements, one named conf and one named conjac
    add.missing.names(ll,c("conf","conjac"))    
  }

  ## NB: There are issues with scope if you pass rho (i.e. the environment), through the parameter list of Call.AR.L (the
  ## elements in the call list for Call.AR.L do not appear be in that environment)
  .Call("we04ucc",psize,a,lower,upper,body(i.objfun),body(i.confun),initial,optlist,new.env())
}

we04ucc2 = function(psize,a,lower,upper,objfun,confun,initial,optlist = NA) {
  ## R code to call e04ucc
  
  ## psize   - Vector containing the problem size, psize = c(N,NCLIN,NCNLIN),
  ##           where N = number of variables, NCLIN = number linear constraints
  ##           NCNLIN = number of non-linear constraints
  ## a       - (NCLIN x N) Matrix of linear constraints. If NCLIN = 0, a is not referenced, 
  ## lower   - List of lower constraint limits. List should contain three elements, the first
  ##           a vector of N limits for simple constraints, the second a vector of NCLIN
  ##           limits for the linear constraints and the third a vector of NCNLIN limits for
  ##           the non-linear constraints
  ## upper   - List of upper constraint limits. This should be in the same form as lower
  ## objfun  - Function that calculates the objective function and its gradients. Needs to
  ##           return a list with two elements, one named "objf" and one named "objgrd" holding
  ##           the value of the objective function and its gradient respectively.
  ##           objfun = function(n,x,flag,first)
  ##               n     - N, the number of parameters
  ##               x     - Vector of length N holding the values at which the objective function
  ##                       should be evaluated
  ##               flag  - If 0 or 2 the value of the objective function should be calculated
  ##                       If 2 the value of the gradient should be calculated
  ##               first - Set to TRUE if this is the first call to the objfun
  ##           If objfun returns a list that includes an entry called "terminate" the routine will stop
  ## confun  - Function that calculates the non-linear constraints and corresponding derivatives.
  ##           Needs to return a list with two elements, one named "conf" and one named "conjac" holding
  ##           the value of the constraints and their gradients respectively.
  ##           confun = function(n,ncnlin,needc,x,flag,first)
  ##              n      - N, the number of parameters
  ##              ncnlin - NCNLIN, the number of non-linear constraints
  ##              needc  - Vector of length NCNLIN, if needc[i] = 1, then values for the ith constraint
  ##                       are required
  ##               x     - Vector of length N holding the values at which the objective function
  ##                       should be evaluated
  ##               flag  - If 0 or 2 the value of the objective function should be calculated
  ##                       If 2 the value of the gradient should be calculated
  ##               first - Set to TRUE if this is the first call to the confun
  ##           If confun returns a list that includes an entry called "terminate" the routine will stop
  ## initial - Vector of N initial values
  ## optlist - List of options. The names of the items in the list should correspond to the names of the
  ##           optional parameters as given in section 11.1 of the documentation for e04ucc.
  ##           i.e. optlist=list(list=TRUE, f_prec = 0.4, print_level = 1), sets the optional parameter
  ##           list to Nag_TRUE, the problem accuracy (f_prec) to 0.4 and the print level to Nag_Soln.
  ##
  ##           For options that are enumerations (as print_level is), an integer code should be used, starting
  ##           from 0. The options are then numbered in the same order as they appear in the "constraints" 
  ##           section of the documentation for that option, i.e. for print_level 0 = Nag_NoPrint, 1 = Nag_Soln,
  ##           2 = Nag_Iter etc.
  ##       
  ##           The following options are not catered for: start,conf,conjac,lambda and print_fun
  ##
  ##           If optlist is given as a list, irrespective of whether it has any valid option in it, then
  ##           additional output that is documented as being returned in the Nag_E04_Opt argument options
  ##           is returned

  ## NB: There are issues with scope if you pass rho (i.e. the environment), through the parameter list of Call.AR.L (the
  ## elements in the call list for Call.AR.L do not appear be in that environment)
  .Call("we04ucc",psize,a,lower,upper,objfun,confun,initial,optlist,new.env())
}


## Example dataset
d.psize=c(4,1,2)
d.a = matrix(c(1,1,1,1),ncol=d.psize[1], nrow=d.psize[2])
d.lower=list(c(1,1,1,1),c(-1e-25),c(-1e-25,25))
d.upper=list(c(5,5,5,5),c(20),c(40,1e25))
d.initial = c(1,5,5,1)

## An example of using the function (with derivatives)
examp.confun.with.deriv = function(n,ncnlin,needc,x,conjac,flag,first) {
  ## Routine to set the constraints for e04ucc example
  ## Derivatives provided
  conf = rep(0.0,ncnlin)

  if (needc[1] > 0) {
    if (flag == 0 || flag == 2) {
      conf[1] = sum(x^2)
    }
    if (flag == 2) {
      conjac[1,1:4] = 2.0 * x[1:4]
    }
  }

  if (needc[2] > 0) {
    if (flag == 0 || flag == 2) {
      conf[2] = prod(x)
    }
    
    if (flag == 2) {
      conjac[2,1] = x[2] * x[3] * x[4]
      conjac[2,2] = x[1] * x[3] * x[4]
      conjac[2,3] = x[1] * x[2] * x[4]
      conjac[2,4] = x[1] * x[2] * x[3]
    }
  }

  list(conf=conf,conjac=conjac)
}

examp.objfun.with.deriv = function(n,x,objgrd,flag,first) {
  ## Routine to calculate the objective function for e04ucc example
  ## Derivatives provided
  objf = 0.0
  if (flag == 0 || flag == 2) {
    objf = x[1] * x[4] * (x[1] + x[2] + x[3]) + x[3]
  }
  
  if (flag == 2) {
    objgrd[1] = x[4] * (2.0*x[1] + x[2] + x[3])
    objgrd[2] = x[1] * x[4]
    objgrd[3] = x[1] * x[4] + 1.0
    objgrd[4] = x[1] * (x[1] + x[2] + x[3])
  }

  list(objf=objf,objgrd=objgrd)
}

## Run the NAG example (default options, using derivatives)
we04ucc1(d.psize,
         d.a,
         d.lower,
         d.upper,
         examp.objfun.with.deriv,
         examp.confun.with.deriv,
         d.initial)

##############################################################################
## Example only supplying some of the derivatives
examp.confun.some.deriv = function(n,ncnlin,needc,x,conjac,flag,first) {
  ## Routine to set the constraints for e04ucc example
  ## Only some derivatives provided
  conf = rep(0.0,ncnlin)

  if (needc[1] > 0) {
    if (flag == 0 || flag == 2) {
      conf[1] = sum(x^2)
    }
    conjac[1,3] = 2.0 * x[3]
  }

  if (needc[2] > 0) {
    if (flag == 0 || flag == 2) {
      conf[2] = prod(x)
    }

    ## Only specify two of the derivatives as an example
    ## We therefore do not change the values of the other elements
    if (flag == 2) {
      conjac[2,2] = x[1] * x[3] * x[4]
      conjac[2,3] = x[1] * x[2] * x[4]
    }
  }
  
  list(conf=conf,conjac=conjac)
}

examp.objfun.no.deriv = function(n,x,objgrd,flag,first) {
  ## Routine to calculate the objective function for e04ucc example
  ## No derivatives provided
  objf = 0.0
  if (flag == 0 || flag == 2) {
    objf = x[1] * x[4] * (x[1] + x[2] + x[3]) + x[3]
  }

  list(objf=objf)
}

## Need to set the optional parameter "derivatives"
optlist=list("con_deriv"=FALSE, "obj_deriv"=FALSE)

## These options can be used to turn off additional printed information:
##optlist=list("list"=FALSE, "print_level"=0, "minor_print_level"=0, "verify_grad"=0)
## These options can be used to send information to a file called "e04_outc.txt"
##optlist=list(outfile="test_e04_out.txt")

## Run the example 
we04ucc1(d.psize,
         d.a,
         d.lower,
         d.upper,
         examp.objfun.no.deriv,
         examp.confun.some.deriv,
         d.initial,
         optlist)

## Use the second way of supplying the callback functions
we04ucc2(d.psize,
         d.a,
         d.lower,
         d.upper,
         examp.objfun.no.deriv,
         examp.confun.some.deriv,
         d.initial,
         optlist)
